#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

{
    "name": "test_field_html_extensions",
    "version": "1.0",
    "depends": ["base"],
    "auto_install": False,
    "installable": True,
}
